#!/usr/bin/env Rscript

today <- format(Sys.time(), "%a_%b_%d")
cmd_args <- commandArgs(trailingOnly = T)

if (length(cmd_args) != 1)
{
  message = "Usage: EthSEQ.R <config_file>"
  cat(paste("[",Sys.time() ,"] ",message,"\n",sep=""))
  q()
}

## Load configuration file
configFile <- cmd_args[1]

source(configFile)
source(paste(source.dir,"Functions.R",sep=""))
verbose = !verbose

## Plot intro message
EthSEQ_version<-0.1
message(paste("Running EthSEQ v",EthSEQ_version,sep=""))
message(paste("Configuration file: ",configFile,sep=""))
message(paste("Working directory: ",out.dir,sep=""))

## Select Model
res = get.Model(model)
model.ped = res[[1]]
model.map = res[[2]]
vcf.file = res[[3]]

bam.files = unique(as.character(read.delim(bam.list,header=F)[,1]))
sample.names = gsub(".bam","",basename(bam.files))
## Check length of names for EIGENSTRAT limit
name.lengths = as.numeric(sapply(sample.names,function(x) length(strsplit(x,"")[[1]])))
if(any(name.lengths>39))
{
  message = paste(sample.names[which(name.lengths>39)],collapse="\n")
  error.message(paste("There samples with ID length >39 characters:\n",message,sep=""))
}
## Execute ASEQ to generate genotypes
table = data.frame(sample=gsub(".bam","",basename(bam.files)),bam=bam.files)
genotype.dir = paste(out.dir,"ASEQGenotypes/",sep="")
if(!file.exists(out.dir))
  system(paste("mkdir ",out.dir),ignore.stdout=verbose,ignore.stderr=verbose)

message(paste("Run ASEQ to generate Dataset genotypes on ",length(bam.files)," samples",sep=""))
aseq.run(bam.files,genotype.dir,mbq,mrq,mdc,vcf.file,cores)

## Create EIGENSTRAT folder
eigenstrat.dir = paste(out.dir,"EIGENSTRAT/",sep="")
if(!file.exists(eigenstrat.dir))
  system(paste("mkdir",eigenstrat.dir),ignore.stdout=verbose,ignore.stderr=verbose)

## Create Dataset PED
message("Create aggregated PED and MAP files")
ped = create.PED(genotype.dir)
write.table(ped[,2:ncol(ped)],paste(out.dir,"Dataset.ped",sep=""),sep="\t",col.names=F,row.names=F,quote=F)
## Remove FFID in Hapmap model 
command = paste("awk -F\"\\t\" '{out=$2; for(i=3;i<=NF;i++){out=out\"\\t\"$i}; print out}' ",model.ped," > ",out.dir,"tmp.ped",sep="")
system(command,ignore.stderr=verbose)
## Merge Dataset and Hapmap model
command = paste("cat ",out.dir,"Dataset.ped ",out.dir,"tmp.ped > ",out.dir,"tmp2.ped",sep="")
system(command,ignore.stderr=verbose)
## Add FFID to aggreagated model and mv to eigenstrat folder
command = paste("awk -F\"\\t\" '{out=NR\"\\t\"$1; for(i=2;i<=NF;i++){out=out\"\\t\"$i}; print out}' ",out.dir,"tmp2.ped > ",eigenstrat.dir,"DatasetHapMap.ped",sep="")
system(command,ignore.stderr=verbose)
## Remove temporary files
system(paste("rm ",out.dir,"Dataset.ped",sep=""),ignore.stderr=verbose)
system(paste("rm ",out.dir,"tmp.ped",sep=""),ignore.stderr=verbose)
system(paste("rm ",out.dir,"tmp2.ped",sep=""),ignore.stderr=verbose)
## Copy HapMap MAP file to eigestrat folder
system(paste("cp ",model.map," ",eigenstrat.dir,"DatasetHapMap.map",sep=""),ignore.stderr=verbose)

## Run EIGENSTRAST
message("Run EIGESTRAT")
setwd(eigenstrat.dir)
config = paste(
  "genotypename: DatasetHapMap.ped",
  "snpname: DatasetHapMap.map",
  "indivname: DatasetHapMap.ped",
  "outputformat: EIGENSTRAT",
  "genotypeoutname: DatasetHapMap.eigenstratgeno",
  "snpoutname: DatasetHapMap.snp",
  "indivoutname: DatasetHapMap.ind",
  "familynames: NO",
sep="\n")
write(config,paste(eigenstrat.dir,"par.PED.EIGENSTRAT",sep=""))
system(paste(eigenstrat.path,"convertf -p par.PED.EIGENSTRAT",sep=""),ignore.stdout=verbose,ignore.stderr=verbose)

config = paste(
  "genotypename: DatasetHapMap.eigenstratgeno",
  "snpname: DatasetHapMap.snp",
  "indivname: DatasetHapMap.ind",
  "evecoutname: DatasetHapMap.pca.evec",
  "evaloutname: DatasetHapMap.eval",
  "altnormstyle: NO",
  "numoutevec: 10",
  "numoutlieriter: 0",
  "numoutlierevec: 2",
  "outliersigmathresh: 6.0",
  sep="\n")
write(config,paste(eigenstrat.dir,"DatasetHapMap.pca.par",sep=""))
system(paste(eigenstrat.path,"smartpca -p DatasetHapMap.pca.par",sep=""),ignore.stdout=verbose,ignore.stderr=verbose)

## Generate reports
message("Generate Analysis Reports")
plot.Report.Model(eigenstrat.dir,length(bam.files))

message("Computation finished")
