## Run ASEQ to get genotype
aseq.run <- function(bam.files,out.dir,mbq,mrq,mdc,vcf.file,cores)
{
  for (b in bam.files)
  {
    command = paste(ASEQ.path," vcf=",vcf.file," bam=",b," mode=GENOTYPE threads=",cores," htperc=0.2 mbq=",mbq,
                    " mrq=",mrq," mdc=",mdc," out=",out.dir,sep="")
    system(command,ignore.stderr=verbose)
    if(verbose)
      cat("*")
  }
  if(verbose)
    cat("\n")
}

## Create PED of ASEQ genotype results
create.PED <- function(genotype.dir)
{
  files = list.files(genotype.dir,"genotype.vcf")
  FID = 1
  for(f in files)
  {
    geno = read.table(paste(genotype.dir,f,sep=""),sep="\t",header=F,as.is=T)
    order = apply(geno,1,function(x) x[4]<x[5])
    geno = geno[,10]
    geno[which(geno=="./.")] = "0 0"
    geno[which(geno=="0/1")] = "1 2"
    geno[which(geno=="0/0"&order)] = "1 1"
    geno[which(geno=="1/1"&order)] = "2 2"
    geno[which(geno=="0/0"&!order)] = "2 2"
    geno[which(geno=="1/1"&!order)] = "1 1"
    if(f==files[1])
    {
      ped = c(FID,gsub(".genotype.vcf","",f),0,0,0,1,geno)
    } else
    {
      ped = rbind(ped,c(FID,gsub(".genotype.vcf","",f),0,0,0,1,geno))
    }
    FID=FID+1
  }
  return(ped)
}

## Plot Report and final table
plot.Report.Model <- function(eigenstrat.dir,n.samples)
{
  library(SDMTools)
  setwd(eigenstrat.dir)
  
  ### Load PCA results
  pca_Allgenome<-read.table("DatasetHapMap.pca.evec", stringsAsFactors=F)

  ### Load SIF Datset
  tab = cbind(pca_Allgenome[,1],NA)
  names = c()
  for(i in 1:nrow(tab))
  {
    if(length(grep(tab[i,1],as.character(table$sample)))>0)
    {
      tab[i,2] = as.character(table$bam[grep(tab[i,1],as.character(table$sample))])
      names = cbind(names,as.character(table$sample[grep(tab[i,1],as.character(table$sample))])==tab[i,1])
    }
  }
  tab = tab[which(!is.na(tab[,2])),]
  if(!all(names))
    error.message("ERROR")
  
  ### Load HapMap description
  idx.hapmap = (n.samples+1):nrow(pca_Allgenome)
  sif = read.table(paste(source.dir,"Models/SIF.txt",sep=""),sep="\t",as.is=T,header=T)

  ### Print PCA schema
  populations = unique(sif$pop)
  colors = rainbow(length(populations),alpha=0.8)
  polygons = list()
  
  cols = rep(rgb(0,0,0,alpha=0.8),nrow(pca_Allgenome))
  for(i in 1:length(populations))
  {
    ids = which(pca_Allgenome[,1]%in%sif$sample[which(sif$pop==populations[i])])
    cols[ids] = colors[i]
    tmp = pca_Allgenome[ids,]
    hpts = chull(tmp[,2],tmp[,3])
    hpts = c(hpts, hpts[1])
    polygons[[length(polygons)+1]] = tmp[hpts,2:3]
  }
  
  tab = as.data.frame(tab)
  tab$V3 = NA
  tab$V4 = NA
  tab$V5 = ""
  tab$V6 = ""
  tab$V7 = "black"
  
  for(i in 1:nrow(tab))
  {
    id = which(pca_Allgenome[,1]==tab[i,1]) 
    tab[i,3] = pca_Allgenome[id,2]
    tab[i,4] = pca_Allgenome[id,3]
    
    for(j in 1:length(polygons))
    {
      polypnts = polygons[[j]] 
      pnts = data.frame(as.numeric(tab[i,3]),as.numeric(tab[i,4]))
      colnames(polypnts) = c("x","y")
      colnames(pnts) = c("x","y")
      rbind(polypnts,pnts)
      res = pnt.in.poly(pnts,polypnts)
      if(res[1,3]==1)
      {
        tab[i,5] = paste(c(strsplit(tab[i,5],"\\|")[[1]],populations[j]),collapse="|")
        tab[i,6] = "INSIDE"
      }
    }
    
    if(tab[i,5]=="")
    {
      ### Find closest point
      tmp = pca_Allgenome[idx.hapmap,1:3]
      vals = apply(tmp[2:3],1,function(x) sqrt((x[1]-as.numeric(tab[i,3]))^2+(x[2]-as.numeric(tab[i,4]))^2))
      k = which(vals==min(vals))
      tab[i,5] = paste(unique(sif$pop[which(sif$sample%in%tmp[k,1])]),collapse="|")
      tab[i,6] = "CLOSEST"
    }
    
    tab[i,7] = colors[which(populations==strsplit(as.character(tab$V5[i]),"\\|")[[1]][1])]
    
  }
  
  pdf("Report.pdf",11, 8)
  
  legend.space = (max(pca_Allgenome[,2])-min(pca_Allgenome[,2]))/7
  plot(NA,xlim=c(min(pca_Allgenome[,2])-legend.space,max(pca_Allgenome[,2])),ylim=c(min(pca_Allgenome[,3]),max(pca_Allgenome[,3])),
       main="Dataset samples with HapMap classes",xlab="PC1",ylab="PC2")
  points(as.numeric(tab[,3]),as.numeric(tab[,4]),pch=16,cex=0.6)
  for(i in 1:length(polygons))
  {
    lines(polygons[[i]][,1],polygons[[i]][,2],lwd=3,col=colors[i])
  }
  legend("bottomleft",legend=populations,pch=16, col=colors,bty = "n",cex=0.7)
  
  plot(NA,xlim=c(min(pca_Allgenome[,2])-legend.space,max(pca_Allgenome[,2])),ylim=c(min(pca_Allgenome[,3]),max(pca_Allgenome[,3])),
       main="Dataset samples",xlab="PC1",ylab="PC2")
  points(as.numeric(tab[,3]),as.numeric(tab[,4]),col=tab$V7,pch=16,cex=0.6)
  legend("bottomleft",legend=populations,pch=16, col=colors,bty = "n",cex=0.7)
  
  plot(NA,xlim=c(min(pca_Allgenome[,2])-legend.space,max(pca_Allgenome[,2])),ylim=c(min(pca_Allgenome[,3]),max(pca_Allgenome[,3])),
       main="HapMap samples",xlab="PC1",ylab="PC2")
  points(pca_Allgenome[idx.hapmap,2],pca_Allgenome[idx.hapmap,3],col=cols[idx.hapmap],pch=16,cex=0.6)
  legend("bottomleft",legend=populations,pch=16, col=colors,bty = "n",cex=0.7)
  
  dev.off()
  
  out.tab = tab[,c(1,2,5,6)]
  colnames(out.tab) = c("sample","bam","ethnicity","type")
  write.table(out.tab,"Report.csv",sep="\t",quote=F,row.names=F)
  
  pdf("Report_Detailed.pdf",11, 8)
  
  for(j in 1:nrow(tab))
  {
    legend.space = (max(pca_Allgenome[,2])-min(pca_Allgenome[,2]))/7
    plot(NA,xlim=c(min(pca_Allgenome[,2])-legend.space,max(pca_Allgenome[,2])),ylim=c(min(pca_Allgenome[,3]),max(pca_Allgenome[,3])),
         main=paste("Sample",tab[j,1]),xlab="PC1",ylab="PC2")
    points(as.numeric(tab[j,3]),as.numeric(tab[j,4]),pch=16,cex=0.8)
    for(i in 1:length(polygons))
    {
      lines(polygons[[i]][,1],polygons[[i]][,2],lwd=3,col=colors[i])
    }
    legend("bottomleft",legend=populations,pch=16, col=colors,bty = "n",cex=0.7)
    text(min(pca_Allgenome[,2])-legend.space,max(pca_Allgenome[,3]),paste("Annotation: ",tab[j,5],sep=""),cex=0.8,adj=0)
  }
  
  dev.off()
  
}

## Set selected model
get.Model <- function(model)
{
  models.available = c("SS2","SS4","HALO")
  if(model=="HALO") 
  {
    model.ped = paste(source.dir,"Models/HapMapAllPopulations.HALO.ped",sep="")  
    model.map = paste(source.dir,"Models/HapMapAllPopulations.HALO.map",sep="") 
    vcf.file = paste(source.dir,"VCF/HapMapAllPopulations.HALO.vcf",sep="") 
  } 
  if(model=="SS4") 
  {
    model.ped = paste(source.dir,"Models/HapMapAllPopulations.SS4.ped",sep="")  
    model.map = paste(source.dir,"Models/HapMapAllPopulations.SS4.map",sep="") 
    vcf.file = paste(source.dir,"VCF/HapMapAllPopulations.SS4.vcf",sep="") 
  } 
  if(model=="SS2") 
  {
    model.ped = paste(source.dir,"Models/HapMapAllPopulations.SS2.ped",sep="")  
    model.map = paste(source.dir,"Models/HapMapAllPopulations.SS2.map",sep="") 
    vcf.file = paste(source.dir,"VCF/HapMapAllPopulations.SS2.vcf",sep="") 
  } 
  if(!model%in%models.available)
  {
    error.message("ERROR: Model not available")
  }
  return(list(model.ped,model.map,vcf.file))
}

## Message
message <- function(message)
{
  cat(paste("[",Sys.time() ,"] ",message,".\n",sep=""))
}

## Error message
error.message <- function(message)
{
  cat(paste("[",Sys.time() ,"] ",message,".\n",sep=""))
  q()
}

